#include "mtl/Sort.h"
#include "core/Solver.h"
//#include "core/Constants.h"
#include "utils/System.h"

using namespace Minisat;

// sattime functions

#define noRedundant -7
#define redundant -77
#define newRedundant -777
#define oldRedundant -7777

// use the nbTrue and lastSatVar slots for redundant clause detecting
// If the new clause is redundant, then return newRedundant, else return 
// the number of redundant old clauses
int Solver::mark_redundant_clause(int cIndex, vec<int>& involved) {
  int i, j, count=0;
  // initialize involved count of each clause to 0
  for(i=0; i<involved.size(); i++)
    clausesLS[involved[i]].setNbTrue(0);
  involved.shrink_(involved.size());
  CRef cr=clauses[cIndex];
  ClauseLS& cls=clausesLS[cIndex];
  Clause& c = ca[cr];
  for(j=0; j<c.size(); j++) {
    // inClauses[c[j]] is a list of no of clauses containing c[j]
    vec<CRefLS>& ks=inClauses[c[j]];
    for(int k=0; k<ks.size(); k++) {
      ClauseLS& kcls=clausesLS[ks[k]];
      if (kcls.lastSatVar() != redundant) {
	if (kcls.nbTrue()==0)
	  involved.push(ks[k]);
	kcls.setNbTrue(kcls.nbTrue()+1);
	if (kcls.nbTrue()==ca[kcls.cr].size()) {
	  cls.setLastSatVar(redundant);
	  return newRedundant;
	}
	if (j==c.size()-1 && kcls.nbTrue()==c.size()) {
	  kcls.setLastSatVar(redundant);
	  count++;
	  //  return oldRedundant;
	}
      } 
    }
  }
  return count;
}

void Solver::attachClauseForSattime(CRefLS i) {
  CRef cr=clauses[i];
  ClauseLS& cls=clausesLS[i];
  cls.setNbTrue(0); cls.setCriVar(0); cls.setLastSatVar(-1);
  cls.setMostRecent(-1); cls.setUnSATindex(-1); cls.cr=cr;
  Clause& c = ca[cr];
  for(int j=0; j<c.size(); j++)
    inClauses[c[j]].push(i);
}

void Solver::clearInClausesLists() {
  int i;
  for(i=0; i<nVars(); i++) {
    Lit p=mkLit(i, false);
    inClauses[p].shrink_(inClauses[p].size());
    inClauses[~p].shrink_(inClauses[~p].size());
  }
}

void Solver::attachClausesForSattime() {
  // int i, j, res, count=0;
  //vec<int> involved;
  // bool redo=false;

  for(int i=0; i<clauses.size(); i++) 
    attachClauseForSattime(i);
}
  /*
  involved.clear();
  for(i=0; i<clauses.size(); i++) {
    res=mark_redundant_clause(i, involved);
    if (res == newRedundant)
      count++;
    else {
      attachClauseForSattime(i);
	count += res;
    // if (res==oldRedundant || res==newRedundant)
    //   redo=true; // if a clause is redundant
    }
  }
  for(i=0; i<involved.size(); i++)
    clausesLS[involved[i]].setNbTrue(0);
  involved.shrink_(involved.size());
  if (count>0) {
    clearInClausesLists();
    // FILE* fout=fopen("myout", "w");
    for(i=j=0; i<clauses.size(); i++) {
      ClauseLS& cls=clausesLS[i];
      if (cls.lastSatVar() == redundant) {
	// output the removed clauses
	// Clause& c=ca[clauses[i]];
	// for(int k=0; k<c.size(); k++)
	//   fprintf(fout, "%d ",  toInt(c[k])%2==0 ? toInt(c[k])/2+1 : -(toInt(c[k])/2+1));
	// fprintf(fout, "                      : %d\n", i);
	removeClause(clauses[i]);
      }
      else {
	clauses[j]=clauses[i];
	attachClauseForSattime(j);
	j++;
      }
    }
    //  fclose(fout);
    printf("c reconstruct inClauses...\n");
    printf("c removed %d redundant clauses over %d clauses with %d fixed vars\n", 
	   i-j, i, trail.size());
    clauses.shrink(i-j);
  }
  else printf("c removed %d redundant clauses over %d clauses with %d fixed vars\n", 
	      count, clauses.size(),  trail.size());
	      }*/

void Solver::clause_value() {
  int i, j, nb, cri; 
  for (i=0; i<clauses.size(); i++) {
    ClauseLS& cls=clausesLS[i];
    Clause& c=ca[cls.cr];
    nb=0; cri=0;
    for(j=0; j<c.size(); j++)
      if (sign(c[j]) == polarity[var(c[j])]) {
	nb++; cri+=var(c[j]);
      }
    cls.setNbTrue(nb); cls.setCriVar(cri);
    if (nb==0) {
      cls.setUnSATindex(unSAT.size());
      unSAT.push(i);
    }
  }
}

void Solver::getNeibors(int v) {
  int i, j;
  MYFLAG++;
  // in minisat, literal is positive with "false" in mkLit 
  // to make the positive literal represented by even number
  Lit p=mkLit(v, false); 
  vec<CRefLS>& poscs=inClauses[p];
  for(i=0; i<poscs.size(); i++) {
    Clause& c=ca[clausesLS[poscs[i]].cr]; 
    for(j=0; j<c.size(); j++) {
      if (v!=var(c[j]) && permDiff[var(c[j])] !=MYFLAG) {
	permDiff[var(c[j])]=MYFLAG;
	neibors[v].push(var(c[j]));
      }
    }
  }
  Lit q=mkLit(v, true);
  vec<CRefLS>& negcs=inClauses[q];
  for(i=0; i<negcs.size(); i++) {
    Clause& c=ca[clausesLS[negcs[i]].cr]; 
    for(j=0; j<c.size(); j++) {
      if (v!=var(c[j]) && permDiff[var(c[j])] !=MYFLAG) {
	permDiff[var(c[j])]=MYFLAG;
	neibors[v].push(var(c[j]));
      }
    }
  }
}

void Solver::getNeibors() {
  int i;

  for(i=0; i<nVars(); i++)
    if (value(i) == l_Undef)
      getNeibors(i);
  MYFLAG=0;
  for(i=0; i<nVars(); i++) 
    permDiff[i]=0;
}

// // p is in each clause in cs
// int Solver::getGradient(Lit& p, vec<CRef>& cs) {
//   int i, nb;
//   nb=0;
//   if (polarity[var(p)]==sign(p))
//     for(i=0; i<cs.size(); i++) {
//       Clause& c=ca[cs[i]];
//       if (c.nbTrue()==1)
// 	nb++;
//     }
//   else 
//     for(i=0; i<cs.size(); i++) {
//       Clause& c=ca[cs[i]];
//       if (c.nbTrue()==0)
// 	nb++;
//     }
//   return nb;
// }

int Solver::getVarScore(int v) {
  int nbP, nbnonP, i;
  Lit p=mkLit(v, polarity[v]); 
  vec<CRefLS>& cs1=inClauses[p]; nbP=0;
  for(i=0; i<cs1.size(); i++) {
    ClauseLS& cls=clausesLS[cs1[i]];
    if (cls.nbTrue()==1)
	nbP++; // nb of clauses satisfied only by p
  }
  vec<CRefLS>& cs2=inClauses[~p]; nbnonP=0;
  for(i=0; i<cs2.size(); i++) {
    ClauseLS& cls=clausesLS[cs2[i]];
    if (cls.nbTrue()==0)
      nbnonP++;//nb of clauses that would become satisfied if p is flipped 
  }
  return nbnonP-nbP;
}

int Solver::random_integer(int max)
{
  unsigned long int RAND;
  RAND=rand();
  return RAND % max;
}

struct timeval tv;
struct timezone tzp;

void Solver::modify_seed() {
  int seed;
  if (SEED_FLAG) {
    srand(SEED); SEED=SEED+17;
    if (SEED==0) SEED=17;
  }
  else {
    gettimeofday(&tv,&tzp);
    seed = (( tv.tv_sec & 0177 ) * 1000000) + tv.tv_usec;
    srand(seed);
  }
}

void Solver::initialize() {
  int i, j, nb, cri;

  modify_seed();
  decrVars.clear();
  freeedVars.clear();
  freeedBetterVars.clear();
  unSAT.clear();
  // compute (and set) the number of satisfied lits, crivar, 
  //and collect unsat clauses
  for (i=0; i<clauses.size(); i++) {
    ClauseLS& cls=clausesLS[i];
    Clause& c=ca[cls.cr];
    nb=0; cri=0;
    for(j=0; j<c.size(); j++)
      if (sign(c[j]) == polarity[var(c[j])]) {
	nb++; cri+=var(c[j]);
      }
    cls.setNbTrue(nb); cls.setCriVar(cri);
    if (nb==0) {
      cls.setUnSATindex(unSAT.size());
      unSAT.push(i);
    }
  }
  // compute the score of each var and collect decVars
  for(i=0; i<nVars(); i++) {
    if (value(i) == l_Undef) {
      flip_time[i]=0;
      tmp_score[i]=0;
      score[i]=getVarScore(i);
      if (score[i]>0)
	decrVars.push(i);
    }
  }
}

int Solver::diversifyForUneven(CRef cr, Var best_v) {
  int flip_index, i, nb;
 
  Clause& c=ca[cr];
  flip_index=random_integer(c.size()-1); nb=0;
  for(i=0; i<c.size(); i++) {
    if (best_v != var(c[i])) {
      if (nb==flip_index)
	return var(c[i]);
      else nb++;
    }
  }
  return best_v;   // not useful, just for avoiding compiling warning
}

int Solver::my_get_var_to_flip_in_clause(CRefLS crls) {
  int v, best_v=NONE, second_best_v, nb, max_nb, second_max, flip_index, i;
  ClauseLS& cls=clausesLS[crls];
  Clause& c=ca[cls.cr];
  if (random_integer(100)<LNOISE) {
    flip_index=random_integer(c.size());
    return var(c[flip_index]);
  }

  max_nb=-clauses.size();
  for(i=0; i<c.size(); i++) {
    v=var(c[i]);
    nb=score[v];
    if ((nb>max_nb) || ((nb==max_nb) && (flip_time[v]<flip_time[best_v]))) {
	second_best_v=best_v; second_max=max_nb; best_v=v; max_nb=nb;
      }
      else if ((nb>second_max) || 
	       ((nb==second_max) && (flip_time[v]<flip_time[second_best_v]))) {
	second_max=nb; second_best_v=v;
      }
  }
  if (tabu_sattime[best_v]==TRUE && best_v==cls.lastSatVar()) {
    if (random_integer(100)<NOISE/10)
      return diversifyForUneven(cls.cr, best_v);
    else if (random_integer(100)<NOISE) 
      return second_best_v; 
    else return best_v;
  }
  else return best_v;
}

int Solver::my_choose_var_by_random_walk() {
  CRefLS crls; int index;
  index=random_integer(unSAT.size());
  crls=unSAT[index];
  return my_get_var_to_flip_in_clause(crls);
}

void Solver::cleanDecVars(vec<Var>& vs) {
  int i, j;
  Var v;
  for (i=0; i<vs.size(); i++) {
    v=vs[i];
    if (score[v]<=0) {
      break;
    }
  }
  if (i<vs.size()) {
    for (j=i+1; j<vs.size(); j++) {
      v=vs[j];
      if (score[v]>0) {
	vs[i++]=v;
      }
    }
    vs.shrink(j-i);
  }
}

int Solver::choose_best_decreasing_var(vec<Var>& vs) {
  int i,  ft;
  Var v, chosen_v;
  chosen_v=vs[0]; ft=flip_time[chosen_v];
  for(i=1; i<vs.size(); i++) {
    v=vs[i];
    if (flip_time[v]<ft) {
      ft=flip_time[v]; chosen_v=v;
    }
  }
  return chosen_v;
}

int Solver::choose_decVar() {

  cleanDecVars(decrVars);
  cleanDecVars(freeedVars);
  cleanDecVars(freeedBetterVars);
  if (decrVars.size()>0)
    return choose_best_decreasing_var(decrVars);
  // else if (freeedVars.size()>0)
  //   return choose_best_decreasing_var(freeedVars);
  // else if (freeedBetterVars.size()>0)
  //   return choose_best_decreasing_var(freeedBetterVars);
  return NONE;
}

#define invPhi 10
#define invTheta 5

void Solver::initNoise() {
  lastAdaptFlip=0;
  lastBest = unSAT.size();
  NOISE=0; LNOISE=0;
  AdaptLength=clauses.size() / invTheta;
}

void Solver::adaptNoveltyNoise(int flip) {

  if ((flip - lastAdaptFlip) > AdaptLength) {
    NOISE += (int) ((100 - NOISE) / invPhi);
    LNOISE= (int) NOISE/10;
    lastAdaptFlip = flip;      
    lastBest = unSAT.size();
  } 
  else if (unSAT.size() < lastBest) {
    NOISE -= (int) (NOISE /(2*invPhi));
    LNOISE= (int) NOISE/10;
    lastAdaptFlip = flip;
    lastBest = unSAT.size();
  }
}

void Solver::my_satisfy_clauses(Var v, vec<CRefLS>& cs) {
  Var neibor_v;
  int i, j;
  for(i=0; i<cs.size(); i++) {
    ClauseLS& cls=clausesLS[cs[i]];
    Clause& c=ca[cls.cr];
    cls.setNbTrue(cls.nbTrue()+1);
    switch(cls.nbTrue()) {
    case 1: {
      cls.setLastSatVar(v);
      int index=cls.unSATindex();
      CRefLS lastCls=unSAT[index]=unSAT.last();
      unSAT.pop();
      clausesLS[lastCls].setUnSATindex(index);
      for(j=0; j<c.size(); j++) {
	neibor_v=var(c[j]);
	if (neibor_v != v) {
	  tabu_sattime[neibor_v]=FALSE;
	  tmp_score[neibor_v]--;
	}
      }
      cls.setCriVar(v);
      break; }
    case 2: {
      tmp_score[cls.criVar()]++;
      cls.setCriVar(cls.criVar()+v);
      break;}
    default: cls.setCriVar(cls.criVar()+v);
    }
  }
}

void Solver::my_unsatisfy_clauses(Var v, vec<CRefLS>& cs) {
  Var neibor_v;
  int i, j;
  for(i=0; i<cs.size(); i++) {
    ClauseLS& cls=clausesLS[cs[i]];
    Clause& c=ca[cls.cr];
    cls.setNbTrue(cls.nbTrue()-1); cls.setCriVar(cls.criVar()-v);
    switch(cls.nbTrue()) {
    case 0:
      cls.setMostRecent(v);
      cls.setUnSATindex(unSAT.size());
      unSAT.push(cs[i]);
      for(j=0; j<c.size(); j++) {
	neibor_v=var(c[j]);
	if (neibor_v != v) {
	  tabu_sattime[neibor_v]=FALSE;
	  tmp_score[neibor_v]++;
	}
      }
      break;
    case 1: 
      tmp_score[cls.criVar()]--;
      break;
    }
  }
}

void Solver::check_implied_clauses(Var v) {
  int i;
  Lit p=mkLit(v, polarity[v]);

  my_satisfy_clauses(v, inClauses[p]);
  my_unsatisfy_clauses(v, inClauses[~p]);
  vec<Var>& vs=neibors[v];
  tabu_sattime[v]=TRUE;
  for(i=0; i<vs.size(); i++) {
    Var neibor_v=vs[i];
    if (score[neibor_v]+tmp_score[neibor_v]>0) {
      if (score[neibor_v]<=0)
	decrVars.push(neibor_v);
      // else if (tabu_sattime[neibor_v] == FALSE)
      // 	freeedVars.push(neibor_v);
      // else if (tmp_score[neibor_v]>0)
      // 	freeedBetterVars.push(neibor_v);
    }
    score[neibor_v]+=tmp_score[neibor_v];
    tmp_score[neibor_v]=0;
  }
}

// void Solver::removeClauseForSattime(CRef cr) {
//   Clause& c=ca[cr];

//   for(int i=0; i<c.size(); i++)
//     inClauses.smudge(c[i]);
// }

#define MAXTRIES 1
#define MAXSTEPS 100000000

int  Solver::sattime(int maxsteps) {
  int nbunsat, i, j;
  Var var_to_flip, v;
  printf("c Soving instance by sattime...\n");

  /* for(i=savedTrailForSattime; i<trail.size(); i++) { */
  /*   Lit p=trail[i]; */
  /*   vec<CRef>& cs=inClauses[p]; */
  /*   for(j=0; j<cs.size(); j++) */
  /*     removeClauseForSattime(cs[j]); */
  /*   vec<Var>& vs=neibors[var(p)]; */
  /*   for(j=0; j<vs.size(); j++) */
  /*     neibors.smudge(vs[j]); */
  /* } */
  /* inClauses.cleanAll(); */
  /* neibors.cleanAll(); */
  /* savedTrailForSattime=trail.size(); //clean from this point the next time */
  double beginSattime=cpuTime();
  if (clausesLS == NULL) 
    clausesLS = (ClauseLS *)malloc(clauses.size()*sizeof(ClauseLS));
  else clearInClausesLists();
  attachClausesForSattime();
  getNeibors();
  double endSattimePreproc=cpuTime();
  printf("c sattime preprocessing time: %12.2fs\n", endSattimePreproc-beginSattime);

  for (i=0;i<MAXTRIES;i++) {
    // compute (and set) the number of satisfied lits, crivar, 
    //and collect unsat clauses
    // compute the score of each var and collect decVars
    initialize();
    initNoise();
    nbunsat=unSAT.size();
    printf("c %d clauses unsat\n", nbunsat);
    if (decrVars.size()>0) {
      int index=random_integer(decrVars.size());
      var_to_flip=decrVars[index];
    }
    else var_to_flip=NONE;
    int tenthSteps = maxsteps/100;
    for (j=0;j<maxsteps;j++) {
      if (j==tenthSteps) {
	double tenthTime=cpuTime()-endSattimePreproc;
	if (tenthTime > 5)
	  maxsteps=(int)((500.0/tenthTime)*tenthSteps);
      }
      //  check();
      if (nbunsat==0) {
	for(v=0;v<nVars();v++)
	  if (value(v) == l_Undef) 
	    assigns[v]=lbool(!((bool) polarity[v]));
	int k, maxSize=0;
	for(k=0; k<clauses.size(); k++) {
	  if (ca[clauses[k]].size() > maxSize)
	    maxSize = ca[clauses[k]].size();
	  if (!satisfied(ca[clauses[k]])) {
	    printf("c clause %d non-satisfied\n", k);
	    break;
	  }
	}
	printf("c solved by sattime in %d flips, maxClauseSize = %d \n", j, maxSize);
	if (k==clauses.size())
	  printf("c solution verified\n");
	double endSattimeSAT=cpuTime();
	printf("c sattime time: %12.2fs, flips: %d\n", endSattimeSAT-beginSattime, j);
	return true;
      }
      if (var_to_flip==NONE) {
	var_to_flip=my_choose_var_by_random_walk();
      }
      polarity[var_to_flip]=!polarity[var_to_flip]; 
      check_implied_clauses(var_to_flip);
      flip_time[var_to_flip]=j;
      adaptNoveltyNoise(j);
      score[var_to_flip]=0-score[var_to_flip];
      var_to_flip=choose_decVar();
      if (unSAT.size()<nbunsat) {
	nbunsat=unSAT.size();
	//	printf("c %d clauses unsat\n", nbunsat);
      }
    }
  }
  double endSattime=cpuTime();
  printf("c sattime time: %12.2fs, flips: %d, nbunsat: %d\n", endSattime-beginSattime, j, nbunsat);
  return false;
}

